close all
clearvars
clc
tic;

turns = 20;
   
%% Material specific parameters

% Surface properties

sites = 50;                  % Number of surface sites = 2*(sites)^2 (use even number!)
lattice_a = 4.74;            % Al2O2

%% Define hexagonal grid

figure('Name','Density',...
'Units', 'centimeters',...
'Position',[0 0 10 8],...
'PaperPositionMode','auto');

x1 = (0:1:sites-1);
x2 = x1;
y1 = x1;
y2 = x1;
x1 = lattice_a*x1;
y1 = sqrt(3)*lattice_a*y1;
x2 = lattice_a*x2+lattice_a/2;
y2 = sqrt(3)*lattice_a*y2 + sqrt(3)*lattice_a/2;
[X1,Y1] = meshgrid(x1,y1);
[X2,Y2] = meshgrid(x2,y2);
X = [X1;X2];
Y = [Y1;Y2];
% scatter(X(:),Y(:),15,'bo')
% hold on
gridpoints = [X(:),Y(:)];

Xmax = max(gridpoints(:,1))+lattice_a/2;
Ymax = max(gridpoints(:,2))+sqrt(3)*lattice_a/2;

Area=Xmax*Ymax;      % Surface area of lattice [A^2]

% Create grid for periodic boundary conditions by mirroring the grid 8
% times around the original grid and then only using the neccessary points closest to the
% edges of the original grid

k = boundary(gridpoints(:,1),gridpoints(:,2));  % Find boundary points main grid

edge_grid = [gridpoints-[Xmax,Ymax];...       % Create mirror grids around main grid
    gridpoints+[Xmax,Ymax];...
    gridpoints-[Xmax,-Ymax];...
    gridpoints+[Xmax,-Ymax];...
    gridpoints-[0,Ymax];...
    gridpoints+[0,Ymax];...
    gridpoints+[Xmax,0];...
    gridpoints-[Xmax,0];...
    ];

Rn = 12;     % Necessary radius depends on the length of the molecule

loc=[];                                         % Find all points within radius from boundary points
for i = 1:1:size(k,1)
    Idx = rangesearch(edge_grid,gridpoints(k(i),:),Rn);
    loc = [loc;transpose(Idx{1,1}(2:end))];                                     
end

edge_grid = unique(edge_grid(loc,:),'rows');    % Every edge grid point is only added to the total grid once
% plot(edge_grid(:,1),edge_grid(:,2),'ro','MarkerSize',4)        % Plot edge grid in green (not necessary)
n = sites;

% Hard boundary
border(:,1) = [-lattice_a/4; -lattice_a/4; Xmax-lattice_a/4;  Xmax-lattice_a/4; -lattice_a/4]; 
border(:,2) = [-sqrt(3)*lattice_a/4; Ymax-sqrt(3)*lattice_a/4; Ymax-sqrt(3)*lattice_a/4; -sqrt(3)*lattice_a/4; -sqrt(3)*lattice_a/4];

Xmax = Xmax*[0,0,-1,1,1,-1,-1,1];                   % These values are used for finding sites on the edge grid
Ymax = Ymax*[1,-1,0,0,1,1,-1,-1];                   % during adsorption on edge sites


%% Extra starting parameters

time = zeros(turns,1);
coverage = zeros(turns,1);
Q_turn = zeros(turns,23,2*sites^2+150);
R_turn = Q_turn;
placed_list_turn = zeros(turns,2*sites^2+150);
config_counter = placed_list_turn;

for turn = 1:1:turns
tic;
close all
clearvars -except border Q1 R1 gridpoints Xmax Ymax Q_turn R_turn time placed_list_turn turn n edge_grid lattice_a config_counter


scatter(gridpoints(:,1),gridpoints(:,2),4,'ko')
hold on
plot(edge_grid(:,1),edge_grid(:,2),'o','Color',[175 175 175]/255,'MarkerSize',2)
daspect([1 1 1])

i = 1;
g=1;
counter = 1;            
edge_counter = 1;
counter_Real = 0;
tot = 23;
Q=zeros(tot,2*n^2);                    
R=Q;
Q_edge = zeros(tot,size(edge_grid,1));
R_edge = Q_edge;

Q_total = [Q,Q_edge];
R_total = Q_total;

Area_molecule = zeros(2*n^2,1);

% Gridpoint lists
randlist = transpose(1:1:2*n^2);  % Create random numbers list
chosenlist = zeros(2*n^2,1);      % Create picked random gridpoints list
placed_list = chosenlist;
edge_chosenlist = chosenlist;
edge_plot = edge_chosenlist;
total_grid = [gridpoints; edge_grid];

% Flags for while loops
flagsim = 1;

%% Choose random coordinate
while flagsim ~= 0
    flagrandom = 1;
    while flagrandom ~= 0
        randnr=randlist(randi(size(randlist,1)));   % Choose random gridpoint
        chosenlist(counter) = randnr; % Add random gridpoint to chosen list
        randlist(randlist==randnr) = [];            % Delete random gridpoint from random numbers list
        X = gridpoints(randnr,1);                   % Register coordinates
        Y = gridpoints(randnr,2);
        
        % Find nearest neighbours of chosen gridpoint for analysis
        Rn = 12;                                         % Nearest neighbour radius
        
        Idx = rangesearch(total_grid,gridpoints(randnr,:),Rn); % Locate nearest neighbours and exclude chosen point itself
        loc = transpose(Idx{1,1}(2:end));              
        [K,L] = ismember(loc,placed_list);               % Find if nearest neighbours have already had adsorption
        L(L==0)=[];                                     % Delete nearest neighbours without adsorption
        p = size(L,1);
        
        % Do same for mirror_grid (boundary condition grid)
        Idx = rangesearch(edge_grid,gridpoints(randnr,:),Rn);
        loc_mirror = transpose(Idx{1,1}(2:end));
        flag_edge = size(loc_mirror,1);
      
        % Choose random configuration
        config_list = [1,2,3,4];
        t = 1;

        %%
        % Place molecule on gridpoint if possible
        flagplacing = 1;
        j = 1;

        while flagplacing ~= 0
            
            config = config_list(t);

            % Draw molecule with random angle
            if config == 1
                rangle = randi(20)*pi/20;
                ranglist = randperm(40)*pi/20;
                rang = rangle + ranglist(j);
                Hacac_chelate_20220429
            end
            
            flagplacing = 0;
            
            for k = 1:p                             % Analyze only nearest neighbours with adsorption
                [xk,yk] = polyxpoly(Q(:,counter),R(:,counter),Q_total(:,L(k,1)),R_total(:,L(k,1))); % Check for overlap
                flagplacing=flagplacing+sum(xk)+sum(yk);    % If overlap is detected flag3 becomes non-zero and the while loop continues rotating the generated molecule
            end
            
%% Hard boundary
%             if flag_edge >= 0
%                 [xk,yk] = polyxpoly(Q(:,counter),R(:,counter),border(:,1),border(:,2)); % Check for overlap
%                 flagplacing=flagplacing+sum(xk)+sum(yk);    % If overlap is detected flag3 becomes non-zero 
%             end
%%
            flagrandom = flagplacing;               % If placement is okay, continue to next step
            j = j + 1;                              % If no placement is possible, get new coordinate
            if j >= 40
                t = t + 1;
                j = 1;
                if t > 2
                    flagplacing = 0;
                    flagrandom = 1;
                    i = i+1;
                end
            end
        end
        if size(randlist,2)==0
            flagrandom = 0;
            flagsim = 0;
        end
    end
    %% 
    if flagsim ~= 0
        
        plot(Q(:,counter),R(:,counter),'b');
        placed_list(counter) = randnr;
        config_counter(turn,counter) = config;
        Q_total(:,counter)=Q(:,counter);
        R_total(:,counter)=R(:,counter);
        counter = counter + 1;
%% Periodic boundary
        if flag_edge >= 0
            X_old = X;
            Y_old = Y;
            dist = zeros(8,1);
            for j = 1:1:size(Xmax,2)
                [k, dist(j)]  = dsearchn(total_grid,[X_old+Xmax(:,j),Y_old+Ymax(:,j)]);
                if dist(j) <= 1
                    X = X_old+Xmax(j);
                    Y = Y_old+Ymax(j);
                    if config == 1
                        Hacac_chelate_20220429
                    end
                    Q_total(:,edge_counter+2*n^2) = Q(:,counter);
                    R_total(:,edge_counter+2*n^2) = R(:,counter);
                    plot(Q_total(:,edge_counter+2*n^2),R_total(:,edge_counter+2*n^2),'Color', [175 175 175]/255);
                    placed_list = [placed_list;k];
                    edge_counter = edge_counter+1;
                end
            end 
        end
%%      Remove nearest neighbours from possible gridpoints
%        Rn = 2*1.5-.01;                               % Radius within neighbours have to lie
        Rn = 2*2-.01;
        Idx = rangesearch(gridpoints,gridpoints(randnr,:),Rn);
        loc = Idx{1,1}(:);
        for j =1:size(Idx{1,1},2)
    %             plot(gridpoints(loc(j),2),gridpoints(loc(j),1),'ro')
                randlist(randlist==Idx{1,1}(j)) = [];                   % Delete random gridpoint from random numbers list
                i = i+1;
        end
        if size(randlist,2) == 0
           flagsim = 0;
        end
    end

end

Q_turn(turn,:,:) = Q_total(:,1:2*n^2+150);
R_turn(turn,:,:) = R_total(:,1:2*n^2+150);
placed_list_turn(turn,1:size(placed_list,1)) = placed_list;
time(turn) = toc;

end

save Hacac_RSA_on_Al2O3_9deg_chelate_20220429.mat
save Hacac_RSA_on_Al2O3_9deg_chelate_20220429_counter.txt counter -ascii
save Hacac_RSA_on_Al2O3_9deg_chelate_20220429_config_counter.txt config_counter -ascii
save Hacac_RSA_on_Al2O3_9deg_chelate_20220429_area.dat Area_molecule -ascii
